@@ -9,16 +9,15 @@
 #' the \code{interview number} variable of a given year in the individual file. Run \code{example(build.panel)} for a demonstration.
 #' For approach one, accepted input data are stata format .dta, .csv files or R data formats .rda and RData. This usage is similar to stata module \code{psiduse}. 
 #' Approach two follows the strategy introduced at \url{http://asdfree.com}. In fact, both approaches use the same function \code{save.psid} to download the data, \code{psidR} automates the merge and subsetting proceedure for you. 
-#' @param datadir either \code{NULL}, in which case saves to tmpdir or path to directory containing family files ("FAMyyyy.xyz") and individual file ("IND2009ER.xyz") in admissible formats .xyz. Admissible are .dta, .csv, .RData, .rda. Please follow naming convention. Only .dta version <= 12 supported. Recommended usage is to specify \code{datadir}.
+#' @param datadir either \code{NULL}, in which case saves to tmpdir or path to directory containing family files ("FAMyyyy.RData") and individual file ("IND2009ER.RData").
 #' @param fam.vars data.frame of variable to retrieve from family files. Can contain see example for required format.
 #' @param ind.vars data.frame of variables to get from individual file. In almost all cases this will be the type of survey weights you want to use. don't include id variables ER30001 and ER30002.
 #' @param wealth.vars data.frame of variables to get from the wealth supplement files.
-#' @param SAScii logical TRUE if you want to directly download data into Rda format (no dependency on STATA/SAS/SPSS). may take a long time, but downloads only once if you specify \code{datadir}.
 #' @param current.heads.only logical TRUE if user wants current household heads only. Distinguishes mover outs heads.
 #' @param heads.only logical TRUE if user wants household heads only. Household heads in sample year.
 #' @param sample string indicating which sample to select: "SRC" (survey research center), "SEO" (survey for economic opportunity), "immigrant" (immigrant sample), "latino" (Latino family sample). Defaults to NULL, so no subsetting takes place.
 #' @param design either character \emph{balanced} or \emph{all} or integer. \emph{balanced} means only individuals who appear in each wave are considered. \emph{All} means all are taken. An integer value stands for minimum consecutive years of participation, i.e. design=3 means present in at least 3 consecutive waves.
-#' @param verbose logical TRUE if you want verbose output.
+#' @param loglevel one of INFO, WARN and DEBUG. INFO by default.
 #' @import SAScii RCurl data.table
 #' @return
 #' \item{data}{resulting \code{data.table}. the variable \code{pid} is the unique person identifier, constructed from ID1968 and pernum.}
@@ -58,14 +57,14 @@
 #' if (wealth) {
 #'   w = dcast(w[,list(year,name,variable)],year~name)
 #'   d = build.panel(datadir="~/datasets/psid/",fam.vars=f,
-#'                  ind.vars=i,wealth.vars=w,SAScii = TRUE, 
-#'                  heads.only =TRUE,sample="SRC",verbose=TRUE,
+#'                  ind.vars=i,wealth.vars=w, 
+#'                  heads.only =TRUE,sample="SRC",
 #'                  design="all")
 #'   save(d,file="~/psid_wealth.RData")
 #' } else {
 #'   d = build.panel(datadir="~/datasets/psid/",fam.vars=f,
-#'                  ind.vars=i,SAScii = TRUE, 
-#'                  heads.only =TRUE,sample="SRC",verbose=TRUE,
+#'                  ind.vars=i, 
+#'                  heads.only =TRUE,sample="SRC",
 #'                  design="all")
 #'   save(d,file="~/psid_no_wealth.RData")
 #' }
@@ -79,7 +78,7 @@
 #' ## make reproducible family data sets for 2 years
 #' ## variables are: family income (Money) and age
 #' 
-#' ## Data acquisition step: you download data or
+#' ## Data acquisition step:
 #' ## run build.panel with sascii=TRUE
 #' 
 #' # testPSID creates artifical PSID data
@@ -111,23 +110,23 @@
 #' 
 #' d <- build.panel(datadir=my.dir,fam.vars=famvars,
 #'                  ind.vars=indvars,
-#'                  heads.only=FALSE,verbose=TRUE)	
+#'                  heads.only=FALSE)	
 #' 
 #' # see what happens if we drop non-heads
 #' # only the ones who are heads in BOTH years 
 #' # are present (since design='balanced' by default)
 #' d <- build.panel(datadir=my.dir,fam.vars=famvars,
 #'                  ind.vars=indvars,
-#'                  heads.only=TRUE,verbose=FALSE)	
-#' print(d$data[order(pid)],nrow=Inf)
+#'                  heads.only=TRUE)	
+#' print(d[order(pid)],nrow=Inf)
 #' 
 #' # change sample design to "all": 
 #' # we'll keep individuals if they are head in one year,
 #' # and drop in the other
 #' d <- build.panel(datadir=my.dir,fam.vars=famvars,
 #'                  ind.vars=indvars,heads.only=TRUE,
-#'                  verbose=FALSE,design="all")	
-#' print(d$data[order(pid)],nrow=Inf)
+#'                  design="all")	
+#' print(d[order(pid)],nrow=Inf)
 #' 
 #' file.remove(paste0(my.dir,"/FAM1985ER.rda"),
 #'             paste0(my.dir,"/FAM1986ER.RData"),
@@ -138,16 +137,18 @@
 #' # #####################################################################
 #' # Please go to https://github.com/floswald/psidR for more example usage
 #' # #####################################################################
-build.panel <- function(datadir=NULL,fam.vars,ind.vars=NULL,wealth.vars=NULL,SAScii=FALSE,heads.only=FALSE,current.heads.only=FALSE,sample=NULL,design="balanced",verbose=FALSE){
+build.panel <- function(datadir=NULL,fam.vars,ind.vars=NULL,wealth.vars=NULL,heads.only=FALSE,current.heads.only=FALSE,sample=NULL,design="balanced",loglevel=INFO){
 	
+  flog.threshold(loglevel)
 
 	# locally bind all variables to be used in a data.table
 	# or R CMD CHECK complains.
 
-	interview <- headyes <- .SD <- fam.interview <- ind.interview <- ind.head <- ER30001 <- ind.head.num <- pid <- ID1968 <- pernum <- isna <- present <- always <- enough <- ind.seq <- NULL
+	interview <- headyes <- .SD <- fam.interview <- ind.interview <- ind.head <- ER30001 <- ind.head.num <- pid <- ID1968 <- pernum <- isna <- present <- always <- enough <- ind.seq <- name <- variable <- NULL
 
 	stopifnot(is.numeric(fam.vars$year))
 	years <- fam.vars$year
+	flog.debug("years:",years,capture=TRUE)
 
 	# figure out platform
 	s <- .Platform$file.sep
@@ -164,237 +165,187 @@ build.panel <- function(datadir=NULL,fam.vars,ind.vars=NULL,wealth.vars=NULL,SAS
 	} else {
 		if (substr(datadir,nchar(datadir),nchar(datadir))!=s) datadir <- paste0(datadir,s)
 	}
+	flog.debug("datadir: %s",datadir)
 
 	# are we processing wealth supplements?
 	any.wealth = is.data.frame(wealth.vars)
+	flog.debug("any.wealth? %d",any.wealth)
+	
 
 	# data acquisition
 	# ----------------
+	lf = list.files(datadir)
+	flog.debug("datadir contains: ",lf,capture=TRUE)
 
-	if (SAScii){
+	wlth.down <- TRUE  # initiate to something
 
-		lf = list.files(datadir)
-	
-		wlth.down <- TRUE  # initiate to something
+	# all psid family files
+	family    <- data.frame(year = c( 1968:1997 , seq( 1999 , 2015 , 2 ) ),file = c( 1056 , 1058:1082 , 1047:1051 , 1040 , 1052 , 1132 , 1139 , 1152  , 1156, 1164 , 1183 ))
+	# family    <- data.frame(year = c( 1968:1997 , seq( 1999 , 2013 , 2 ) ),file = c( 1056 , 1058:1082 , 1047:1051 , 1040 , 1052 , 1132 , 1139 , 1152  , 1156, 1164  ))
 
-		# all psid family files
-		family    <- data.frame(year = c( 1968:1997 , seq( 1999 , 2015 , 2 ) ),file = c( 1056 , 1058:1082 , 1047:1051 , 1040 , 1052 , 1132 , 1139 , 1152  , 1156, 1164 , 1183 ))
-		# family    <- data.frame(year = c( 1968:1997 , seq( 1999 , 2013 , 2 ) ),file = c( 1056 , 1058:1082 , 1047:1051 , 1040 , 1052 , 1132 , 1139 , 1152  , 1156, 1164  ))
+	#subset to the years we want
+	family <- family[family$year %in% years, ]
+	flog.debug("family df:",family,capture=TRUE)
 
-		#subset to the years we want
-		family <- family[family$year %in% years, ]
-
-		families.down <- rep(FALSE,nrow(family))
-		for ( i in 1:nrow( family )) {
-			if ((paste0("FAM" , family[ i , 'year' ], "ER.rda") %in% lf)) {
-				families.down[i] <- TRUE
-				cat('found ',paste0("FAM" , family[ i , 'year' ], "ER.rda"), 'already downloaded \n')
-			}
+	families.down <- rep(FALSE,nrow(family))
+	for ( i in 1:nrow( family )) {
+		if ((paste0("FAM" , family[ i , 'year' ], "ER.rda") %in% lf) | (paste0("FAM" , family[ i , 'year' ], "ER.RData") %in% lf)) {
+			families.down[i] <- TRUE
+			flog.info('found %s already downloaded',paste0("FAM" , family[ i , 'year' ], "ER.rda"))
 		}
-
-		if (any.wealth){
-			# if any of 1984, 1989, 1994, 1999, 2001, 2003, 2005, 2007 in years, also download the associated wealth supplement
-			wlth = data.frame(year=c(1984, 1989, 1994, 1999, 2001, 2003, 2005, 2007),file=c(1147,1148,1149,1150,1130,1131,1133,1140))
-			if (verbose){
-				cat("years: \n")
-				print(years)
-				cat("wealth.vars$year: \n")
-				print(wealth.vars$year)
-				cat("wlth: \n")
-				print(wlth)
-			}
-			wlth = wlth[wealth.vars$year %in% years, ]
-
-			if ( nrow(wlth) > 0 ){
-			  wlth.down <- rep(FALSE,nrow(wlth))
-			    for ( i in 1:nrow( wlth )) {
-			      if ((paste0("WEALTH" , wlth[ i , 'year' ], "ER.rda") %in% lf)) {
-			        wlth.down[i] <- TRUE
-			        cat('found ',paste0("WEALTH" , wlth[ i , 'year' ], "ER.rda"), 'already downloaded \n')
-			      } else {
-			        cat('will download as ',paste0("WEALTH" , wlth[ i , 'year' ], "ER.rda"), '\n')
-			      }
-			    }
-  			} else {
-  			  wlth.down = TRUE
-  			  cat('All Wealth files already downloaded. \n')
-  			  
-  			  # saying we already downloaded even if we didn't look for wealth vars.
-  			}
-			
-		}
-
-		ind.down <- FALSE
-		# check if datadir contains individual index already
-		if (("IND2015ER.rda" %in% lf)) {
-			#download latest individual index
-			ind.down = TRUE
-		}
-		if (all(all(families.down),ind.down,all(wlth.down))) {
-			cat("everything already downloaded. Build dataset now.\n")
-		} else {
-			cat("Will download missing datasets now:\n")
-			if (!all(families.down)) {
-				cat("will download: \n",family[!families.down,"year"],'\n')
-			} 
-			if (!ind.down) {
-				cat("will download: IND2015ER \n")
-			} 
-			if (!wlth.down) {
-				cat("will download missing wealth files. \n")
-			}
-
-	    	confirm <- readline("This can take several hours/days to download.\n want to go ahead? give me 'yes' or 'no'.")
-			if (confirm=="yes"){
-
-				ftype <- "Rdata"
-				
-				user <- readline("please enter your PSID username: ")
-				pass <- readline("please enter your PSID password: ")
-				
-				curl = getCurlHandle()
-				curlSetOpt(cookiejar = 'cookies.txt', followlocation = TRUE, autoreferer = TRUE, curl = curl)
-
-				html <- getURL('http://simba.isr.umich.edu/u/Login.aspx', curl = curl)
-
-				viewstate <- as.character(sub('.*id="__VIEWSTATE" value="([0-9a-zA-Z+/=]*).*', '\\1', html))
-
-				# extract the `eventvalidation` string
-				eventvalidation <- 
-					as.character(
-						sub(
-							'.*id="__EVENTVALIDATION" value="([0-9a-zA-Z+/=]*).*' , 
-							'\\1' , 
-							html
-						)
-					)
-
-				# construct a list full of parameters to pass to the umich website
-				params <- 
-					list(
-						'ctl00$ContentPlaceHolder1$Login1$UserName'    = user ,
-						'ctl00$ContentPlaceHolder1$Login1$Password'    = pass ,
-						'ctl00$ContentPlaceHolder1$Login1$LoginButton' = 'Log In' ,
-						'__VIEWSTATE'                                  = viewstate ,
-						'__EVENTVALIDATION'                            = eventvalidation
-				    )
-
-				#file number 1053 is always the individual cross year index file
-				#it must always be the last file in this list.
-				#you always want to download that.
-
-				for ( i in 1:nrow( family )) {
-					if (!(paste0("FAM" , family[ i , 'year' ], "ER.rda") %in% lf)) {
-						get.psid( family[ i , 'file' ] ,name= paste0(datadir, "FAM" , family[ i , 'year' ], "ER") , params , curl )
-					}
-				}
-
-				if (nrow(wlth)>0){
-				  for ( i in 1:nrow(wlth)){
-				    if (!(wlth.down[i])){
-				      get.psid( wlth[ i , 'file' ] ,name= paste0(datadir, "WEALTH" , wlth[ i , 'year' ], "ER") , params , curl )
-				    }
-				  }
-				}
-				
-
-				# check if datadir contains individual index already
-				if (!("IND2015ER.rda" %in% lf)) {
-					#download latest individual index
-					get.psid( 1053 ,name= paste0(datadir, "IND2015ER") , params , curl )
-				}
-
-				cat('finished downloading files to', datadir,'\n')
-				cat('continuing now to build the dataset.\n')
-							
-			} else if (confirm=="no") {
-				return(0)
-			}
-		}  # end download data
 	}
 
+	if (any.wealth){
+		# if any of 1984, 1989, 1994, 1999, 2001, 2003, 2005, 2007 in years, also download the associated wealth supplement
+		wlth = data.frame(year=c(1984, 1989, 1994, 1999, 2001, 2003, 2005, 2007),file=c(1147,1148,1149,1150,1130,1131,1133,1140))
+		flog.debug("working on wealth")
+		wlth = wlth[wealth.vars$year %in% years, ]
+		flog.debug("wlth: ",wlth, capture=TRUE)
 
-	# work out file types
-	# -------------------
+		if ( nrow(wlth) > 0 ){
+		  wlth.down <- rep(FALSE,nrow(wlth))
+		    for ( i in 1:nrow( wlth )) {
+		      if ((paste0("WEALTH" , wlth[ i , 'year' ], "ER.rda") %in% lf) | (paste0("WEALTH" , wlth[ i , 'year' ], "ER.RData") %in% lf)) {
+		        wlth.down[i] <- TRUE
+		        flog.info('found %s already downloaded',paste0("WEALTH" , wlth[ i , 'year' ]))
+		      } else {
+		        flog.info('will download as %s',paste0("WEALTH" , wlth[ i , 'year' ], "ER.rda"))
+		      }
+		    }
+			} else {
+			  wlth.down = TRUE
+			  flog.info('All Wealth files already downloaded')
+			  
+			  # saying we already downloaded even if we didn't look for wealth vars.
+			}
+		
+	}
+
+	ind.down <- FALSE
+	# check if datadir contains individual index already
+	if (("IND2015ER.rda" %in% lf) | ("IND2015ER.RData" %in% lf)) {
+		#download latest individual index
+		ind.down = TRUE
+	}
+	if (all(all(families.down),ind.down,all(wlth.down))) {
+		flog.info("everything already downloaded. Build dataset now")
+	} else {
+		flog.info("Will download missing datasets now")
+		if (!all(families.down)) {
+			flog.info("will download family files: %s",paste(family[!families.down,"year"],collapse=", "))
+		} 
+		if (!ind.down) {
+			flog.info("will download: IND2015ER")
+		} 
+		if (!wlth.down) {
+			flog.info("will download missing wealth files.")
+		}
+
+    	confirm <- readline("This can take several hours/days to download.\n want to go ahead? give me 'yes' or 'no'.")
+		if (confirm=="yes"){
+
+			ftype <- "Rdata"
+			
+			user <- readline("please enter your PSID username: ")
+			pass <- readline("please enter your PSID password: ")
+			
+			curl = getCurlHandle()
+			curlSetOpt(cookiejar = 'cookies.txt', followlocation = TRUE, autoreferer = TRUE, curl = curl)
+
+			html <- getURL('http://simba.isr.umich.edu/u/Login.aspx', curl = curl)
+
+			viewstate <- as.character(sub('.*id="__VIEWSTATE" value="([0-9a-zA-Z+/=]*).*', '\\1', html))
+
+			# extract the `eventvalidation` string
+			eventvalidation <- 
+				as.character(
+					sub(
+						'.*id="__EVENTVALIDATION" value="([0-9a-zA-Z+/=]*).*' , 
+						'\\1' , 
+						html
+					)
+				)
+
+			# construct a list full of parameters to pass to the umich website
+			params <- 
+				list(
+					'ctl00$ContentPlaceHolder1$Login1$UserName'    = user ,
+					'ctl00$ContentPlaceHolder1$Login1$Password'    = pass ,
+					'ctl00$ContentPlaceHolder1$Login1$LoginButton' = 'Log In' ,
+					'__VIEWSTATE'                                  = viewstate ,
+					'__EVENTVALIDATION'                            = eventvalidation
+			    )
+
+			#file number 1053 is always the individual cross year index file
+			#it must always be the last file in this list.
+			#you always want to download that.
+
+			for ( i in 1:nrow( family )) {
+				if (!(paste0("FAM" , family[ i , 'year' ], "ER.rda") %in% lf)) {
+					get.psid( family[ i , 'file' ] ,name= paste0(datadir, "FAM" , family[ i , 'year' ], "ER") , params , curl )
+				}
+			}
+      if (any.wealth){
+        if (nrow(wlth)>0){
+          for ( i in 1:nrow(wlth)){
+            if (!(wlth.down[i])){
+              get.psid( wlth[ i , 'file' ] ,name= paste0(datadir, "WEALTH" , wlth[ i , 'year' ], "ER") , params , curl )
+            }
+          }
+        }
+      }
+			
+			# check if datadir contains individual index already
+			if (!("IND2015ER.rda" %in% lf)) {
+				#download latest individual index
+				get.psid( 1053 ,name= paste0(datadir, "IND2015ER") , params , curl )
+			}
+
+			flog.info('finished downloading files to %s', datadir)
+			flog.info('continuing now to build the dataset')
+						
+		} else if (confirm=="no") {
+			return(0)
+		}
+	}  # end download data
 
-	# figure out filestypes in datadir
 	l <- list.files(datadir)
 	if (length(l)==0) stop('there is something wrong with the data directory. please check path')
 	for (i in 1:length(l)) {
-  		if (tail(strsplit(l[i],"\\.")[[1]],1) == "dta") {ftype   <- "stata"; break}
   		if (tail(strsplit(l[i],"\\.")[[1]],1) == "rda") {ftype   <- "Rdata"; break}
   		if (tail(strsplit(l[i],"\\.")[[1]],1) == "RData") {ftype <- "Rdata"; break}
-  		if (tail(strsplit(l[i],"\\.")[[1]],1) == "csv") {ftype   <- "csv"; break}
 	}
-	if (!(exists("ftype"))) stop("No .dta, .rda, .RData or .csv files found in directory\n you need to either download data with option `SAScii=TRUE`, or put downloaded and processed stata files in to datadir.")
+	if (!(exists("ftype"))) stop("No .rda or .RData files found in directory\n you need to download data first.")
 
-	if (verbose) cat('psidR: Loading Family data\n')
-	if (ftype=="stata"){
-		fam.dat  <- paste0(datadir,grep("FAM",l,value=TRUE,ignore.case=TRUE))
-        fam.dat  <- grep(paste(years,collapse="|"),fam.dat,value=TRUE)
-		tmp <- grep("IND",l,value=TRUE,ignore.case=TRUE)
-		if (length(tmp)>1) {
-		    warning(cat("Warning: you have more than one IND file in your datadir.\nI take the last one:",tail(tmp,1),"\n"))
-			ind.file <- paste0(datadir,tail(tmp,1))	# needs to be updated with next data delivery.
-		} else {
-			ind.file <- paste0(datadir,grep("IND",l,value=TRUE,ignore.case=TRUE))	# needs to be updated with next data delivery.
-		}
-		# TODO
-		# maybe should use 
-		# ind      <- read.dta13(file=ind.file)
-		# if version 13?
-		ind      <- read.dta(file=ind.file)
-		ind.dict <- data.frame(code=names(ind),label=attr(ind,"var.labels"))
-		ind      <- data.table(ind)
-	} else if (ftype=="Rdata") {
-		# familiy data downloaded directly into a dataframe
-		fam.dat  <- paste0(datadir,grep("FAM",l,value=TRUE,ignore.case=TRUE))
-		fam.dat  <- grep(paste(years,collapse="|"),fam.dat,value=TRUE)
+	flog.info('psidR: Loading Family data from .rda files')
+	# familiy data downloaded directly into a dataframe
+	fam.dat  <- paste0(datadir,grep("FAM",l,value=TRUE,ignore.case=TRUE))
+	fam.dat  <- grep(paste(years,collapse="|"),fam.dat,value=TRUE)
 
-		# wealth data downloaded directly into a dataframe
-		if (any.wealth){
-			wlth.dat  <- paste0(datadir,grep("WEALTH",l,value=TRUE,ignore.case=TRUE))
-			wlth.dat  <- grep(paste(years,collapse="|"),wlth.dat,value=TRUE)
-		}
-
-		# individual index
-		tmp <- grep("IND",l,value=TRUE,ignore.case=TRUE)
-		if (length(tmp)>1) {
-		    warning(cat("Warning: you have more than one IND file in your datadir.\nI take the last one:",tail(tmp,1),"\n"))
-			ind.file <- paste0(datadir,tail(tmp,1))	# needs to be updated with next data delivery.
-		} else {
-			ind.file <- paste0(datadir,grep("IND",l,value=TRUE,ignore.case=TRUE))	# needs to be updated with next data delivery.
-		}
-		tmp.env  <- new.env()
-		load(file=ind.file,envir=tmp.env)
-		ind      <- get(ls(tmp.env),tmp.env)	# assign loaded dataset a new name
-		setnames(ind,names(ind), sapply(names(ind), toupper))	## convert all column names to uppercase
-		ind.dict <- NULL
-		ind      <- data.table(ind)
-
-	} else if (ftype=="csv") {
-		fam.dat  <- paste0(datadir,grep("FAM",l,value=TRUE,ignore.case=TRUE))
-		fam.dat  <- grep(paste(years,collapse="|"),fam.dat,value=TRUE)
-		tmp <- grep("IND",l,value=TRUE,ignore.case=TRUE)
-		if (length(tmp)>1) {
-		    warning(cat("Warning: you have more than one IND file in your datadir.\nI take the last one:",tail(tmp,1),"\n"))
-			ind.file <- paste0(datadir,tail(tmp,1))	# needs to be updated with next data delivery.
-		} else {
-			ind.file <- paste0(datadir,grep("IND",l,value=TRUE,ignore.case=TRUE))	# needs to be updated with next data delivery.
-		}
-		ind      <- fread(input=ind.file)
-		ind.dict <- NULL
+	# wealth data downloaded directly into a dataframe
+	if (any.wealth){
+		wlth.dat  <- paste0(datadir,grep("WEALTH",l,value=TRUE,ignore.case=TRUE))
+		wlth.dat  <- grep(paste(years,collapse="|"),wlth.dat,value=TRUE)
 	}
+
+	# individual index
+	tmp <- grep("IND",l,value=TRUE,ignore.case=TRUE)
+	if (length(tmp)>1) {
+	    flog.warn("Warning: you have more than one IND file in your datadir. I take the last one:",tail(tmp,1))
+		ind.file <- paste0(datadir,tail(tmp,1))	# needs to be updated with next data delivery.
+	} else {
+		ind.file <- paste0(datadir,grep("IND",l,value=TRUE,ignore.case=TRUE))	# needs to be updated with next data delivery.
+	}
+	tmp.env  <- new.env()
+	load(file=ind.file,envir=tmp.env)
+	ind      <- get(ls(tmp.env),tmp.env)	# assign loaded dataset a new name
+	setnames(ind,names(ind), sapply(names(ind), toupper))	## convert all column names to uppercase
+	ind      <- data.table(ind)
 	
-	if (verbose){
-		cat('psidR: loaded individual file:',ind.file,'\n')
-		cat('psidR: total memory load in MB:\n')
-		vvs = ceiling(object.size(ind)/1024^2)
-		print(as.numeric(vvs))
-	}
 
-	# data dictionaries
-	fam.dicts <- vector("list",length(years))
+	flog.info('psidR: loaded individual file: %s',ind.file)
+	vvs = ceiling(object.size(ind)/1024^2)
+	flog.info("psidR: total memory load in MB: %d",as.numeric(vvs))
 
 	# output data.tables
 	datas <- vector("list",length(years))
@@ -402,12 +353,11 @@ build.panel <- function(datadir=NULL,fam.vars,ind.vars=NULL,wealth.vars=NULL,SAS
 	# make an index of interview numbers for each year
 	ids <- makeids()
 	w_ids <- makeids.wealth()
-	if (verbose){
-		cat('psidR: here is the list of hardcoded PSID variables\n')
-		cat('psidR: The merge is based on equal values in ind.interview and fam.interview\n')
-		print(ids)
-	}
 
+		flog.debug('psidR: here is the list of hardcoded PSID variables')
+		flog.debug('psidR: The merge is based on equal values in ind.interview and fam.interview')
+		flog.debug("ids:",ids,capture=TRUE)
+	
 	# convert fam.vars to data.table
 	stopifnot(is.data.frame(fam.vars))
 	fam.vars <- data.table(fam.vars)
@@ -430,8 +380,6 @@ build.panel <- function(datadir=NULL,fam.vars,ind.vars=NULL,wealth.vars=NULL,SAS
 		wealth.vars[,interview := w_ids[wealth.vars][,interview]]
 	}
 
-
-	
 	# which vars to keep from ind.files?
 	if (!is.null(ind.vars))	stopifnot(is.list(ind.vars))
 
@@ -441,19 +389,17 @@ build.panel <- function(datadir=NULL,fam.vars,ind.vars=NULL,wealth.vars=NULL,SAS
 	# loop over years
 	for (iy in 1:length(years)){
 		
-		if (verbose) {
-			cat('=============================================\n')
-			cat('psidR: currently working on data for year',years[iy],'\n')
-		}
+			flog.info('psidR: currently working on data for year %d',years[iy])
+		
  
    		# keeping only relevant columns from individual file
 		# subset only if requested.
 		curr <- ids[list(years[iy])]
 		if (years[iy] == 1968){
-		  # there is no sequence variable
-		  ind.subsetter <- as.character(curr[,list(ind.interview,ind.head)])	# keep from ind file
+			# there is no sequence variable
+			ind.subsetter <- as.character(curr[,list(ind.interview,ind.head)])	# keep from ind file
 		} else {
-		  ind.subsetter <- as.character(curr[,list(ind.interview,ind.seq,ind.head)])	# keep from ind file
+			ind.subsetter <- as.character(curr[,list(ind.interview,ind.seq,ind.head)])	# keep from ind file
 		}
 		def.subsetter <- c("ER30001","ER30002")	# must keep those in all years
 
@@ -463,24 +409,40 @@ build.panel <- function(datadir=NULL,fam.vars,ind.vars=NULL,wealth.vars=NULL,SAS
 		# the default column order is: 
 		# "ER30001","ER30002", "current year interview var", "current year sequence number", "current year head indicator"
 		# get a character vector from ind.vars with variable names for this year
-		ind.vars.yr <- ind.vars[list(years[iy]),which(names(ind.vars)!="year"),with=FALSE]
+		ind.nas <- NULL
+		if (!is.null(ind.vars)){
+			ind.vars.yr <- ind.vars[list(years[iy]),which(names(ind.vars)!="year"),with=FALSE]
+			flog.debug("ind.vars.yr:",ind.vars.yr,capture=TRUE)
 
-		# issue https://github.com/floswald/psidR/issues/4
-		# ------------------------------------------------
-		# check for NA in ind.vars: these are years when a certain variable isn not available in the individual index file.
-		# adjust for first year (1968) when `sequence` was not available
-		ind.nas <- subset(ind.vars.yr,select=names(ind.vars.yr)[is.na(ind.vars.yr)])
-		yind    <- copy(ind[,c(def.subsetter,c(ind.subsetter,as.character(subset(ind.vars.yr,select=names(ind.vars.yr)[!is.na(ind.vars.yr)])))),with=FALSE])	
-		# add NA columns
-		if (length(ind.nas)>0){
-		    yind[,(names(ind.nas)) := NA]
+			# issue https://github.com/floswald/psidR/issues/4
+			# ------------------------------------------------
+			# check for NA in ind.vars: these are years when a certain variable isn not available in the individual index file.
+			# adjust for first year (1968) when `sequence` was not available
+			ind.notnas <- NULL
+			if (any(is.na(ind.vars.yr))){
+				ind.nas <- ind.vars.yr[,which(is.na(ind.vars.yr)),with=FALSE]
+				flog.debug("ind.nas:",ind.nas,capture=TRUE)
+			}
+			if (any(!is.na(ind.vars.yr))){
+				ind.notnas <- ind.vars.yr[,which(!is.na(ind.vars.yr)),with=FALSE]
+				flog.debug("ind.notnas:",ind.notnas,capture=TRUE)
+				yind    <- copy(ind[,c(def.subsetter,c(ind.subsetter,as.character(ind.notnas))),with=FALSE])	
+			} else {
+				yind    <- copy(ind[,c(def.subsetter,c(ind.subsetter)),with=FALSE])	
+			}
+			# add NA columns
+			if (!is.null(ind.nas)){
+			    yind[,(as.character(ind.nas)) := NA]
+			}
+		} else {
+			yind <- copy(ind[,c(def.subsetter,c(ind.subsetter)),with=FALSE])	
 		}
+
 		if (years[iy]==1968){
 		    yind[,sequence := NA]
 		    setcolorder(yind,c(1:3,ncol(yind),4:(ncol(yind)-1)))
 		}
 
-
     
     	# sample selection
     	# ----------------
@@ -492,31 +454,27 @@ build.panel <- function(datadir=NULL,fam.vars,ind.vars=NULL,wealth.vars=NULL,SAS
     		if (sample == "SRC"){
 			   n    <- nrow(yind)
 			   yind <- copy(yind[ER30001<3000])	# individuals 1-2999 are from SRC sample
-			   if (verbose){
-				   cat('full',years[iy],'sample has',n,'obs\n')
-				   cat(sprintf('you selected %d obs belonging to %s \n',nrow(yind),sample))
-			   }
+			   
+			   flog.info('full %d sample has %d obs',years[iy],n)
+			   flog.info('you selected %d obs belonging to %s',nrow(yind),sample)
+			   
     		} else if (sample == "SEO"){
 			   n    <- nrow(yind)
 			   yind <- copy(yind[ER30001<7000 & ER30001>5000])
-			   if (verbose){
-				   cat('full',years[iy],'sample has',n,'obs\n')
-				   cat(sprintf('you selected %d obs belonging to %s \n',nrow(yind),sample))
-			   }
+			   
+			   flog.info('full %d sample has %d obs',years[iy],n)
+			   flog.info('you selected %d obs belonging to %s',nrow(yind),sample)
+			   
     		} else if (sample == "immigrant"){
 			   n    <- nrow(yind)
 			   yind <- copy(yind[ER30001<5000 & ER30001>3000])	# individuals 1-2999 are from SRC sample
-			   if (verbose){
-				   cat('full',years[iy],'sample has',n,'obs\n')
-				   cat(sprintf('you selected %d obs belonging to %s \n',nrow(yind),sample))
-			   }
+			   flog.info('full %d sample has %d obs',years[iy],n)
+			   flog.info('you selected %d obs belonging to %s',nrow(yind),sample)
     		} else if (sample == "latino"){
 			   n    <- nrow(yind)
 			   yind <- copy(yind[ER30001<9309 & ER30001>7000])	# individuals 1-2999 are from SRC sample
-			   if (verbose){
-				   cat('full',years[iy],'sample has',n,'obs\n')
-				   cat(sprintf('you selected %d obs belonging to %s \n',nrow(yind),sample))
-			   }
+			   flog.info('full %d sample has %d obs',years[iy],n)
+			   flog.info('you selected %d obs belonging to %s',nrow(yind),sample)
     		}
     	}
 
@@ -528,17 +486,15 @@ build.panel <- function(datadir=NULL,fam.vars,ind.vars=NULL,wealth.vars=NULL,SAS
 		# otherwise a head who died between last and this wave is still head, so there would be two heads in that family.
 		# https://psidonline.isr.umich.edu/Guide/FAQ.aspx?Type=ALL#150
 		if (current.heads.only) {
-		   n    <- nrow(yind)
-		   if (years[iy]==1968){
-		     yind <- yind[,headyes := (yind[,curr[,ind.head],with=FALSE]==curr[,ind.head.num])]
-		   } else {
-		     yind <- yind[,headyes := (yind[,curr[,ind.head],with=FALSE]==curr[,ind.head.num]) & (yind[,curr[,ind.seq],with=FALSE]== 1)]
-		   }
-		   yind <- copy(yind[headyes==TRUE])
-		   if (verbose){
-			   cat('dropping non-current-heads leaves',nrow(yind),'obs\n')
-		   }
-		   yind[,headyes := NULL]
+			n    <- nrow(yind)
+			if (years[iy]==1968){
+				yind <- yind[,headyes := (yind[,curr[,ind.head],with=FALSE]==curr[,ind.head.num])]
+			} else {
+				yind <- yind[,headyes := (yind[,curr[,ind.head],with=FALSE]==curr[,ind.head.num]) & (yind[,curr[,ind.seq],with=FALSE]== 1)]
+			}
+			yind <- copy(yind[headyes==TRUE])
+			flog.info('dropping non-current-heads leaves %d obs',nrow(yind))
+			yind[,headyes := NULL]
 		} else if (heads.only){
 			# https://psidonline.isr.umich.edu/Guide/FAQ.aspx?Type=ALL#250
 			# To create a single year Head/Wife file: 
@@ -551,16 +507,16 @@ build.panel <- function(datadir=NULL,fam.vars,ind.vars=NULL,wealth.vars=NULL,SAS
 		     yind <- yind[,headyes := (yind[,curr[,ind.head],with=FALSE]==curr[,ind.head.num]) & ((yind[,curr[,ind.seq],with=FALSE]> 0) & (yind[,curr[,ind.seq],with=FALSE]< 21))]
 		   }
 		   yind <- copy(yind[headyes==TRUE])
-		   if (verbose){
-			   cat('dropping non-heads leaves',nrow(yind),'obs\n')
-		   }
+		   
+		   flog.info('dropping non-heads leaves %d obs',nrow(yind))
+		   
 		   yind[,headyes := NULL]
 
 		}
 
 		
 
-		if (length(ind.nas)>0){
+		if (!is.null(ind.nas)){
 		    setnames(yind,c("ID1968","pernum","interview","sequence","relation.head",
 		                  (names(ind.vars)[-1])[-which(names(ind.vars)[-1] %in% names(ind.nas))],
 		                  names(ind.nas)))  
@@ -577,27 +533,16 @@ build.panel <- function(datadir=NULL,fam.vars,ind.vars=NULL,wealth.vars=NULL,SAS
 		# ==================================
 
 		# load data for current year, make data dictionary for subsets and save data as data.table
-		if (ftype=="stata") {
-			tmp             <- read.dta(file=fam.dat[iy])
-			fam.dicts[[iy]] <- data.frame(code=names(tmp),label=attr(tmp,"var.labels"))
-			tmp             <- data.table(tmp)
-		} else if (ftype=="csv") {
-			tmp <- fread(input=fam.dat[iy])
-			fam.dicts[[iy]] <- NULL
-		} else if (ftype=="Rdata") {
-			rm(list=ls(envir=tmp.env),envir=tmp.env)
-		   	load(file=fam.dat[iy],envir=tmp.env)
-			tmp             <- get(ls(tmp.env),tmp.env)	# assign loaded dataset a new name
-			fam.dicts[[iy]] <- NULL
-			tmp             <- data.table(tmp)
-		}
+		rm(list=ls(envir=tmp.env),envir=tmp.env)
+	   	load(file=fam.dat[iy],envir=tmp.env)
+		tmp             <- get(ls(tmp.env),tmp.env)	# assign loaded dataset a new name
+		tmp             <- data.table(tmp)
+		
+		vs = ceiling(object.size(tmp)/1024^2)
+		flog.debug('loaded family file: ',fam.dat[iy])
+		flog.debug('current memory load in MB: %d',vs)
+
 
-		if (verbose){
-			cat('loaded family file:',fam.dat[iy],'\n')
-			cat('current memory load in MB:\n')
-			vs = ceiling(object.size(tmp))
-			print(vs,units="Mb")
-		}
 
 		# convert all variable names to lower case in both fam.vars and data file
 		curvars <- fam.vars[list(years[iy]),which(names(fam.vars)!="year"),with=FALSE]
@@ -625,10 +570,6 @@ build.panel <- function(datadir=NULL,fam.vars,ind.vars=NULL,wealth.vars=NULL,SAS
 			setkey(tmp,interview)
 		}
 		
-		if (years[iy] == 1996){
-		  print("yes")
-		}
-		
 		# merge family and yind
 		m <- copy(tmp[yind])
 		m[,year := years[iy] ]
@@ -645,23 +586,24 @@ build.panel <- function(datadir=NULL,fam.vars,ind.vars=NULL,wealth.vars=NULL,SAS
 			   	load(file=iw,envir=tmp.env)
 				tmp             <- get(ls(tmp.env),tmp.env)	# assign loaded dataset a new name
 				tmp             <- data.table(tmp)
+				flog.debug("wealth tmp: ",head(tmp),capture=TRUE)
 			
 				# convert all variable names to lower case in both fam.vars and data file
-				if (verbose) {
-					print(wealth.vars)
-				}
 				curvars <- wealth.vars[list(years[iy]),which(names(wealth.vars)!="year"),with=FALSE]
-				tmpnms = tolower(as.character(curvars))
-				for (i in 1:length(tmpnms)){
-					curvars[[i]] <- tmpnms[i]
-				}
+				flog.debug("wealth curvars: ",curvars,capture=TRUE)
+				curvars[,name := tolower(name)]
+				curvars[,variable := tolower(variable)]
+				curvars[,interview := tolower(interview)]
+
 				setnames(tmp,tolower(names(tmp)))
+				flog.debug("wealth tmp: ",head(tmp),capture=TRUE)
 			
-				curnames <- names(curvars)
 				# current set of variables
-				codes <- as.character(curvars)
+				codes <- c(curvars[,variable],curvars[,interview][[1]])
+				flog.debug("wealth codes: ",codes,capture=TRUE)
 				tmp   <- copy(tmp[,codes,with=FALSE])
-				setnames(tmp,curnames)
+				setnames(tmp,c(curvars[,name],"interview"))
+				flog.debug("wealth tmp: ",head(tmp),capture=TRUE)
 				setkey(tmp,interview)
 				
 				# merge m and wealthfile
@@ -680,7 +622,7 @@ build.panel <- function(datadir=NULL,fam.vars,ind.vars=NULL,wealth.vars=NULL,SAS
 		datas[[iy]] <- copy(m)
 
 
-	}
+	}  # end year
 	
 	data2 <- rbindlist(datas,use.names=TRUE,fill=TRUE)	# glue together
 	rm(datas)
@@ -696,13 +638,13 @@ build.panel <- function(datadir=NULL,fam.vars,ind.vars=NULL,wealth.vars=NULL,SAS
 		data2[,always := max(present) == length(years),by=pid]
 		data2 <- copy(data2[always==TRUE])
 		data2[,always := NULL]
-		if (verbose) cat("balanced design reduces sample from",n,"to",nrow(data2),"\n")
+		flog.info("balanced design reduces sample from %d to %d",n,nrow(data2))
 	} else if (is.numeric(design)){
 		n <- nrow(data2)
 		data2[,enough := max(present) >= design,by=pid]
 		data2 <- copy(data2[enough==TRUE])
 		data2[,enough := NULL]
-		if (verbose) cat("design choice reduces sample from",n,"to",nrow(data2),"\n")
+		flog.info("design choice reduces sample from %d to %d",n,nrow(data2))
 	} else if (design=="all"){
 		# do nothing
 	}
@@ -710,15 +652,92 @@ build.panel <- function(datadir=NULL,fam.vars,ind.vars=NULL,wealth.vars=NULL,SAS
 
 	#     setkey(datas,pid,year)
 	#     rm(ind)
-	out <- list(data=data2,dict=fam.dicts)
-	if (verbose){
-		cat('\n\n\nEnd of build.panel(verbose=TRUE)\n\n\n')
-		cat('==================================\n')
-	}
-
-	return(out)
+	flog.info("End of build.panel")
+	return(data2)
 }
 
+#' one year test, no ind file
+#' 
+#' @param dd Data Dictionary location. If NULL, 
+#' use temp dir and force download
+#' @export
+small.test.noind <- function(dd=NULL){
+ 	cwf = openxlsx::read.xlsx(system.file(package="psidR","psid-lists","psid.xlsx"))
+ 	head_age_var_name <- getNamesPSID("ER17013", cwf, years=c(2003))
+ 	famvars = data.frame(year=c(2003),age=head_age_var_name)
+ 	build.panel(fam.vars=famvars,datadir=dd)
+}
+
+#' one year test, ind file
+#' 
+#' @param dd Data Dictionary location. If NULL, 
+#' use temp dir and force download
+#' @export
+small.test.ind <- function(dd=NULL){
+ 	cwf = openxlsx::read.xlsx(system.file(package="psidR","psid-lists","psid.xlsx"))
+ 	head_age_var_name <- getNamesPSID("ER17013", cwf, years=c(2003))
+ 	educ = getNamesPSID("ER30323",cwf,years=2003)
+ 	famvars = data.frame(year=c(2003),age=head_age_var_name)
+ 	indvars = data.frame(year=c(2003),educ=educ)
+ 	build.panel(fam.vars=famvars,ind.vars=indvars,datadir=dd)
+}
+
+#' three year test, ind file
+#' 
+#' @param dd Data Dictionary location. If NULL, 
+#' use temp dir and force download
+#' @export
+medium.test.ind <- function(dd=NULL){
+	cwf = openxlsx::read.xlsx(system.file(package="psidR","psid-lists","psid.xlsx"))
+	head_age_var_name <- getNamesPSID("ER17013", cwf, years=c(2003,2005,2007))
+ 	educ = getNamesPSID("ER30323",cwf,years=c(2003,2005,2007))
+	famvars = data.frame(year=c(2003,2005,2007),age=head_age_var_name)
+	indvars = data.frame(year=c(2003,2005,2007),educ=educ)
+	build.panel(fam.vars=famvars,ind.vars=indvars,datadir=dd)
+}
+
+#' three year test, no ind file
+#' 
+#' @param dd Data Dictionary location
+#' @export
+medium.test.noind <- function(dd=NULL){
+  cwf = openxlsx::read.xlsx(system.file(package="psidR","psid-lists","psid.xlsx"))
+  head_age_var_name <- getNamesPSID("ER17013", cwf, years=c(2003,2005,2007))
+  famvars = data.frame(year=c(2003,2005,2007),age=head_age_var_name)
+  build.panel(fam.vars=famvars,datadir=dd)
+}
+
+#' three year test, ind file and one NA variable
+#' 
+#' @param dd Data Dictionary location. If NULL, 
+#' use temp dir and force download
+#' @export
+medium.test.ind.NA <- function(dd=NULL){
+	cwf = openxlsx::read.xlsx(system.file(package="psidR","psid-lists","psid.xlsx"))
+	head_age_var_name <- getNamesPSID("ER17013", cwf, years=c(2003,2005,2007))
+ 	educ = getNamesPSID("ER30323",cwf,years=c(2003,2005,2007))
+ 	educ[2] = NA
+	famvars = data.frame(year=c(2003,2005,2007),age=head_age_var_name)
+	indvars = data.frame(year=c(2003,2005,2007),educ=educ)
+	build.panel(fam.vars=famvars,ind.vars=indvars,datadir=dd,loglevel = DEBUG)
+}
+
+#' three year test, ind file and one NA variable and wealth
+#' 
+#' @param dd Data Dictionary location. If NULL, 
+#' use temp dir and force download
+#' @export
+medium.test.ind.NA.wealth <- function(dd=NULL){
+	cwf = openxlsx::read.xlsx(system.file(package="psidR","psid-lists","psid.xlsx"))
+	head_age_var_name <- getNamesPSID("ER17013", cwf, years=c(2005,2007))
+ 	educ = getNamesPSID("ER30323",cwf,years=c(2005,2007))
+ 	educ[2] = NA
+	r = system.file(package="psidR")
+    w = fread(file.path(r,"psid-lists","wealthvars-small.txt"))
+	famvars = data.frame(year=c(2005,2007),age=head_age_var_name)
+	indvars = data.frame(year=c(2005,2007),educ=educ)
+	build.panel(fam.vars=famvars,ind.vars=indvars,wealth.vars=w,datadir=dd,loglevel = DEBUG)
+}
 
 
 #' Build example PSID
@@ -755,13 +774,13 @@ build.psid <- function(datadr="~/datasets/psid/",small=TRUE,wealth=FALSE){
   f = dcast(f[,list(year,name,variable)],year~name)
   if (wealth) {
     w = dcast(w[,list(year,name,variable)],year~name)
-    d = build.panel(datadir=datadr,fam.vars=f,ind.vars=i,wealth.vars=w,SAScii = TRUE, heads.only = TRUE,sample="SRC",verbose=TRUE,design="all")
+    d = build.panel(datadir=datadr,fam.vars=f,ind.vars=i,wealth.vars=w, heads.only = TRUE,sample="SRC",design="all")
     save(d,file="~/psid_wealth.RData")
   } else {
-    d = build.panel(datadir=datadr,fam.vars=f,ind.vars=i,SAScii = TRUE, heads.only = TRUE,sample="SRC",verbose=TRUE,design="all")
+    d = build.panel(datadir=datadr,fam.vars=f,ind.vars=i, heads.only = TRUE,sample="SRC",design="all")
     save(d,file="~/psid_no_wealth.RData")
   }
-	return(d$data)
+	return(d)
 }
 
 
@@ -812,7 +831,7 @@ getNamesPSID <- function(aname, cwf, years = NULL){
         yearkeep <- paste0("Y", years)
         yearkeep <- yearkeep[yearkeep %in% colnames(cwf)]
     }
-    ovalue <- cwf[myvar[1], yearkeep, drop = FALSE]
-    ovalue
+    ovalue <- transpose(cwf[myvar[1], yearkeep, drop = FALSE])
+    ovalue$V1
 }
 
